!= deepconv/arare 絤ή׻Ѽץ
!
!= deepconv/arare main program for moist atmospheric convection
!
! Authors::   SUGIYAMA Ko-ichiro, ODAKA Masatsugu
! Version::   $Id: arare.F90,v 1.1 2011-04-05 14:46:54 sugiyama Exp $
! Tag Name::  $Name: arare4-20110603 $
! Copyright:: Copyright (C) GFD Dennou Club, 2006. All rights reserved.
! License::   See COPYRIGHT[link:../../COPYRIGHT]
!

program arare
  !
  ! ϳإǥ deepconv/arare 絤ή׻Ѽץ.
  !

  ! ⥸塼 ; use statement 
  !

  ! gtool5 Ϣ 
  ! gtool5 modules
  !
  use dc_types,      only: STRING
  use dc_string,     only: StoA
  use dc_message,    only: MessageNotify
  use gtool_historyauto, only: HistoryAutoPut

  ! ⥸塼
  ! Initialize module
  !
  use argset,        only: argset_init  
  use fileset,       only: fileset_init, InitFile
  use debugset,      only: debugset_init
  use timeset,       only: timeset_init, DelTimeLong, DelTimeShort,  &
    &                      NstepDisp, NstepLong, NstepShort    
  use gridset,       only: gridset_init, DimXMin, DimXMax, DimZMin, DimZMax, &
    &                      SpcNum, FileXMin, FileXMax, FileZMin, FileZMax
  use basicset,      only: basicset_init, SpcWetSymbol, &
    &                      xza_MixRtBasicZ, xz_DensBasicZ, xz_EffMolWtBasicZ, &
    &                      xz_PotTempBasicZ, xz_TempBasicZ, xz_PressBasicZ, &
    &                      xz_VelSoundBasicZ, xz_ExnerBasicZ 
  
#ifdef MPI  
  !  MPI ν
  use mpi_wrapper,   only : MPIWrapperInit, MPIWrapperFinalize, myrank
#endif
  
  ! ̷׻⥸塼 
  ! Chemical calculation modules
  !
  use ChemCalc,      only: ChemCalc_init
  use chemdata,      only: chemdata_init

  ! ϳز׻Ѵؿ⥸塼
  ! Dynamical processes module
  !
  use DynFunc,       only: xz_AdvScalar, xz_AdvKm, xza_AdvScalar, &
    &                      pz_AdvVelX, xr_Buoy, xr_AdvVelZ, pz_GradPi
  use DynImpFunc,    only: xz_Exner_init, xz_Exner, xr_GradPi
  
  ! ήȻ׻ѥ⥸塼
  ! Turbulent diffusion module
  !
  use Turbulence,    only: Turbulence_Init, xz_TurbScalar, xza_TurbScalar,  &
    &                      pz_TurbVelX, xr_TurbVelZ, xz_ShearKm, xz_DispKm, &
    &                      xz_DispHeat
  
  ! Υեå׻ѥ⥸塼
  ! Surface flux module
  !
  use HeatFlux, only: &
!   &                 xz_HeatFluxBulk, xz_MixRtFluxBulk, &
    &                 xz_HeatFluxDiff,  xza_MixRtFluxDiff

  ! Ͷ׻ѥ⥸塼
  ! Radiative forceing module
  !
  use Radiation,     only: Radiation_init, xz_RadHeatConst  

  ! ׻ѥ⥸塼
  ! Moist processes modules
  !
  use moistset,      only: moistset_init
  use MoistAdjust,   only: MoistAdjustSvapPress, MoistAdjustNH4SH
  use WarmRainPrm,   only: WarmRainPrm_Init, xz_Rain2GasHeat,            &
    &                      xza_Rain2GasNH4SH, xz_Rain2GasHeatNH4SH,      &
    &                      xza_Rain2Gas, xza_Cloud2Rain, xza_FallRain
  use MoistBuoyancy, only: MoistBuoy_Init, xz_BuoyMoistKm, xr_BuoyMolWt, &
    &                      xr_BuoyDrag
  use fillnegative,  only: FillNegative_init, xza_FillNegative_xza

  ! ٤η׻
  ! Static stability calculation module
  !
  use ECCM,          only: ECCM_Stab

  ! ͳȻ/໤׻ѥ⥸塼
  ! Numerical diffussion /dumping module
  !
  use NumDiffusion,  only: NumDiffusion_Init, xz_NumDiffScalar, &
    &                      xz_NumDiffKm, xza_NumDiffScalar,     &
    &                      pz_NumDiffVelX, xr_NumDiffVelZ
  use damping,       only: damping_init, DampSponge_xz, DampSponge_pz, &
    &                      DampSponge_xr

  use average, only:  xz_avr_pz, xz_avr_xr

  ! ѻʹ⥸塼
  ! Monitor variables setup modules
  !
  use StorePotTemp,  only: StorePotTemp_init, StorePotTempClean, &
    &                      StorePotTempCond
  use StoreMixRt,    only: StoreMixRt_init, StoreMixRtClean, StoreMixRtCond, &
    &                      StoreMixRtFill1,  StoreMixRtFill2
  use StoreBuoy,     only: StoreBuoy_init, StoreBuoyClean
  use StoreStab,     only: StoreStab_init, StoreStabClean
  use StoreMom,      only: StoreMom_init, StoreMomClean

  ! եϥ⥸塼
  ! File I/O module
  !
  use RestartFileIO, only: ReStartFile_Open, ReStartFile_OutPut, &
    &                      ReStartFile_Close, ReStartFile_Get
  use HistoryFileIO, only: HistoryFile_Open, HistoryFile_OutPut, &
    &                      HistoryFile_Close
  use HistoryAutoFileIO, only: HistoryFileAutoOpen, HistoryFileAutoClose

  ! ⥸塼
  ! Utility modules
  !
  use cflcheck,      only: CFLCheckTimeShort, CFLCheckTimeLongVelX, &
    &                      CFLCheckTimeLongVelZ
  use timefilter,    only: AsselinFilter_xz, AsselinFilter_xr, &
    &                      AsselinFilter_pz, AsselinFilter_xza
#ifdef MPI
  use boundaryMPI,      only: BoundaryXCyc_xz, BoundaryZSym_xz,  &
    &                      BoundaryXCyc_xza, BoundaryZSym_xza, BoundaryXCyc_pz,  &
    &                      BoundaryZSym_pz, BoundaryXCyc_xr, BoundaryZAntiSym_xr
#else
  use boundary,      only: BoundaryXCyc_xz, BoundaryZSym_xz,  &
    &                      BoundaryXCyc_xza, BoundaryZSym_xza, BoundaryXCyc_pz,  &
    &                      BoundaryZSym_pz, BoundaryXCyc_xr, BoundaryZAntiSym_xr
#endif

  ! ѿ ; Variables definition 
  !

  implicit none

  ! ѿ
  ! Internal variables
  !
  character(80) :: cfgfile  ! NAMELIST ե̾ NAMELIST file name
  real(8), allocatable :: pz_VelXBl(:,:) 
                            ! $ u(t-\Delta t) $ ʿ Horizontal wind
  real(8), allocatable :: pz_VelXNl(:,:) 
                            ! $ u(t) $ ʿ Horizontal wind
  real(8), allocatable :: xz_VelXNl(:,:) 
                            ! $ u(t) $ ʿ Horizontal wind
  real(8), allocatable :: pz_VelXAl(:,:) 
                            ! $ u (t+\Delta t)a $ ʿ Horizontal wind
  real(8), allocatable :: pz_VelXNs(:,:) 
                            ! $ u (\tau) $ ʿ Horizontal wind
  real(8), allocatable :: pz_VelXAs(:,:) 
                            ! $ u (\tau +\Delta \tau) ʿ Horizontal wind
  real(8), allocatable :: xr_VelZBl(:,:) 
                            ! $ w (t-\Delta t) ľ Vertical wind
  real(8), allocatable :: xr_VelZNl(:,:) 
                            ! $ w (t) ľ Vertical wind
  real(8), allocatable :: xz_VelZNl(:,:) 
                            ! $ w (t) ľ Vertical wind
  real(8), allocatable :: xr_VelZAl(:,:) 
                            ! $ w (t+\Delta t) ľ Vertical wind
  real(8), allocatable :: xr_VelZNs(:,:) 
                            ! $ w (\tau) ľ Vertical wind
  real(8), allocatable :: xr_VelZAs(:,:) 
                            ! $ w (\tau +\Delta \tau) $ ľ Vertical wind
  real(8), allocatable :: xz_ExnerBl(:,:)
                            ! $ \pi (t-\Delta t) $ ϴؿ Exner function
  real(8), allocatable :: xz_ExnerNl(:,:)
                            ! $ \pi (t) $ ϴؿ Exner function
  real(8), allocatable :: xz_ExnerAl(:,:)
                            ! $ \pi (t+\Delta t) $ ϴؿ Exner function
  real(8), allocatable :: xz_ExnerNs(:,:)
                            ! $ \pi (\tau) $ ϴؿ Exner function
  real(8), allocatable :: xz_ExnerAs(:,:)
                            ! $ \pi (\tau +\Delta \tau) $ ϴؿ Exner function
  real(8), allocatable :: xz_PotTempBl(:,:) 
                            ! $ \theta (t-\Delta t) $  Potential temp.
  real(8), allocatable :: xz_PotTempNl(:,:) 
                            ! $ \theta (t) $  Potential temp.
  real(8), allocatable :: xz_PotTempAl(:,:) 
                            ! $ \theta (t+\Delta t) $  Potential temp.
  real(8), allocatable :: xz_KmBl(:,:)
                            ! $ Km (t-\Delta t) $ ήȻ 
                            ! Turbulent diffusion coeff. 
  real(8), allocatable :: xz_KmNl(:,:)         
                            ! $ K_m (t) ήȻ 
                            ! Turbulent diffusion coeff. 
  real(8), allocatable :: xz_KmAl(:,:)
                            ! $ K_m (t+\Delta t) $ ήȻ 
                            ! Turbulent diffusion coeff. 
  real(8), allocatable :: xz_KhBl(:,:)
                            ! $ K_h (t-\Delta t) $ ήȻ 
                            ! Turbulent diffusion coeff. 
  real(8), allocatable :: xz_KhNl(:,:)
                            ! $ K_h (t) $ ήȻ 
                            ! Turbulent diffusion coeff. 
  real(8), allocatable :: xz_KhAl(:,:)
                            ! $ K_h (t+\Delta t) $ ήȻ 
                            ! Turbulent diffusion coeff.
  real(8), allocatable :: xza_MixRtBl(:,:,:)
                            ! $ q (t-\Delta t) $ ̤κ 
                            ! Mixing ratio of moist variables.
  real(8), allocatable :: xza_MixRtNl(:,:,:) 
                            ! $ q (t) $ ̤κ 
                            ! Mixing ratio of moist variables.
  real(8), allocatable :: xza_MixRtAl(:,:,:)
                            ! $ q (t+\Delta t) $ ̤κ 
                            ! Mixing ratio of moist variables.
  real(8), allocatable :: pz_AccelVelXNl(:,:)  
                            ! Ϥ $u$ ѲΨ
                            ! Tendency of $u$ except for pressure gradient term
  real(8), allocatable :: xr_AccelVelZNl(:,:)  
                            ! Ϥ $w$ ѲΨ
                            ! Tendency of $w$ except for pressure gradient term
  real(8) :: Time           !  Time 
  real(8) :: ReStartTime(2) ! ꥹȥեϻ
                            ! Output time array for restart file
  real(8), allocatable :: DelTimeLFrog(:)
                            ! ꡼ץեåѻֳʻҴֳ
                            ! Time interval for Leap-frog scheme
  real(8) :: DelTimeEular   ! 顼ѻֳʻ
                            ! Time interval for Eular scheme
  integer :: NStepLFrog     ! ꡼ץեåѻ֥ƥå׿
                            ! The number of time step for Leap-frog scheme
  integer, allocatable :: NStepEular(:)
                            ! 顼ѻ֥ƥå׿
                            ! The number of time step for Eular scheme
  real(8), allocatable :: pz_VelXPGrd(:,:)
  real(8), allocatable :: xr_VelZPGrd(:,:)
  real(8), allocatable :: pz_VelXTend(:,:)
  real(8), allocatable :: xr_VelZTend(:,:)
  real(8), allocatable :: xz_KmTend(:,:)
  real(8), allocatable :: xz_PotTempTend(:,:)
  real(8), allocatable :: xz_PotTempTendRad(:,:)
  real(8), allocatable :: xz_PotTempTendFlux(:,:)
  real(8), allocatable :: xza_MixRtTend(:,:,:)
  real(8), allocatable :: xza_MixRtTendFlux(:,:,:)
  real(8), allocatable :: xz_PotTempAdvBZ(:,:)
  real(8), allocatable :: xz_PotTempTurbBZ(:,:)
  real(8), allocatable :: xza_MixRtAdvBZ(:,:,:)
  real(8), allocatable :: xza_MixRtTurbBZ(:,:,:)

  integer :: & 
    & t,     & 
    & tau,   & ! do 롼ѿ ; do loop variable  
    & t1,    & ! do 롼ѿ ; do loop variable  
    & t2       ! do 롼ѿ ; do loop variable 

  integer :: s
  logical :: FlagCondKesslerNH4SH = .false. 
  logical :: FlagRadHeatConst = .false. 
  logical :: FlagFluxDiff = .false. 


  ! ³ Initialize procedure 
  !
  call MainInit

  ! ʬ time integration 
  !
  do t1 = 1, NstepLFrog / NstepDisp
    do t2 = 1, NstepDisp
      
      !  
      ! Time setting.
      !
      Time = Time + DelTimeLong
      t = (t1 - 1) * NstepDisp + t2
!      write(*,*) Time

      !------------------------------------
      ! ȻΰήȻ
      ! Advection and diffusion of turbulent diffusion coefficient.
      !
      call TendencyKm_Adv_Diff
      xz_KmAl = xz_KmBl + DelTimeLFrog(t) * xz_KmTend 

      ! ͤξ²¤
      ! * ͤˤʤ뤳Ȥݾڤ
      ! * ͤξ¤ 800 Ȥ. (1994, ذʸ)
      !
      ! Upper and lower bound value are specified.
      !
      xz_KmAl = max( 0.0d0, min( xz_KmAl, 800.0d0 ) )

      !  Boundary condition
      !
      call BoundaryXCyc_xz( xz_KmAl )  ! (inout)
      call BoundaryZSym_xz( xz_KmAl )  ! (inout)

      ! 顼Ф뱲Ȼη׻ 
      ! Specify turbulent diffusion coefficient for scalar variables.
      !
      xz_KhAl = 3.0d0 * xz_KmAl

      !------------------------------------
      ! ̤λȯŸ
      !
      ! ̤ΰήȻη׻ 
      ! Advection and diffusion of potential temperature.
      !
      call TendencyPotTemp_Adv_Diff

      ! ʪ: Ͷ
      ! Radiation of potential temperature
      !
      call TendencyPotTemp_Radiation

      ! ʪ: Ǯ͢
      ! Flux of potential temperature from bottom boundary.
      !
      call TendencyPotTemp_Flux

      ! ʬ
      xz_PotTempAl = xz_PotTempBl   &
        &            + DelTimeLFrog(t) * (xz_PotTempTend + xz_PotTempTendRad + xz_PotTempTendFlux)
      
      !  Boundary condition
      !
      call BoundaryXCyc_xz( xz_PotTempAl ) ! (inout)
      call BoundaryZSym_xz( xz_PotTempAl ) ! (inout)

      !------------------------------------      
      ! ŷʬλȯŸ
      !
      ! ŽʬΰήȻ 
      ! Advection and diffusion of vapor, cloud and rain mixing ratios.
      !
      call TendencyMixRt_Adv_Diff

      ! ŽʬβΥեå
      ! Flux of vapor, cloud and rain mixing ratios from bottom boundary.
      !
      call TendencyMixRt_Flux
      
      ! ʬ
      xza_MixRtAl = xza_MixRtBl &
        &           + DelTimeLFrog(t) * ( xza_MixRtTend + xza_MixRtTendFlux )

      !  Boundary condition
      !
      call BoundaryXCyc_xza( xza_MixRtAl ) ! (inout)
      call BoundaryZSym_xza( xza_MixRtAl ) ! (inout)

      ! ήˤäˤʤäʬ
      ! Negative values due to advection are corrected.
      !
      call AdjustMixRt_FillNegative 

      ! ŷ: Kessler 
      ! 
      call AdjustMixRt_Kessler
      
      !  Boundary condition
      !
      call BoundaryXCyc_xz( xz_PotTempAl ) ! (inout)
      call BoundaryZSym_xz( xz_PotTempAl ) ! (inout)
      call BoundaryXCyc_xza( xza_MixRtAl ) ! (inout)
      call BoundaryZSym_xza( xza_MixRtAl ) ! (inout)
     
      !------------------------------------      
      ! ®٤ΰήȻ.
      ! Advection and diffusion of velocity components.
      !
      call TendencyVelX_Adv_Diff
      call TendencyVelZ_Adv_Diff
      
      ! û֥ƥåפνͺ.
      ! Initial values set up for time integration with short time step.
      !
      pz_VelXNs  = pz_VelXBl
      xr_VelZNs  = xr_VelZBl
      xz_ExnerNs = xz_ExnerBl
      
      ! û֥ƥåפλʬ. 顼ˡ.
      ! Time integration with short time step.
      !
      Eular: do tau = 1, NstepEular(t)

        !-------------------------------------------
        ! ® u η׻.
        ! Time integration horizontal velocity (u).
        !
        call TendencyVelX_PGrd
        pz_VelXAs = pz_VelXNs + DelTimeEular * (pz_VelXPGrd + pz_AccelVelXNl)
        
        !  Boundary condition
        !
        call BoundaryXCyc_pz( pz_VelXAs ) ! (inout)
        call BoundaryZSym_pz( pz_VelXAs ) ! (inout)

        !-------------------------------------------        
        ! ʡؿη׻.
        ! Time integration exner function.
        !
        xz_ExnerAs = xz_Exner( &
          & xr_AccelVelZNl,    &
          & pz_VelXNs,         &
          & pz_VelXAs,         &
          & xr_VelZNs,         &
          & xz_ExnerNs)
        
        !  Boundary condition
        !
        call BoundaryXCyc_xz( xz_ExnerAs ) ! (inout)
        call BoundaryZSym_xz( xz_ExnerAs ) ! (inout)

        !-------------------------------------------        
        ! ® w η׻
        ! Time integration vertical velocity.
        !
        call TendencyVelZ_PGrd
        xr_VelZAs = xr_VelZNs + DelTimeEular * (xr_VelZPGrd + xr_AccelVelZNl)
        
        !  Boundary condition
        !
        call BoundaryXCyc_xr( xr_VelZAs )     ! (iuout)
        call BoundaryZAntiSym_xr( xr_VelZAs ) ! (inout)
        
        ! û֥ƥåפΥ롼פ󤹤ν
        ! Renew prognostic variables for next short time step integration.
        !
        xz_ExnerNs  = xz_ExnerAs
        pz_VelXNs   = pz_VelXAs
        xr_VelZNs   = xr_VelZAs
        
      end do Eular
      
      ! ǽŪû֥ƥåפǤͤĹ֥ƥåפǤͤȤߤʤ
      ! Renew prognostic variables for next long time step integration.
      !
      xz_ExnerAl  = xz_ExnerAs
      pz_VelXAl   = pz_VelXAs
      xr_VelZAl   = xr_VelZAs
      
      ! ֥ե륿. 
      ! Time filter. 
      !
      call AsselinFilter_xz( &
        &  xz_ExnerAl,       & ! (in)
        &  xz_ExnerNl,       & ! (inout)
        &  xz_ExnerBl        & ! (in)
        & )         
      call AsselinFilter_pz( &
        & pz_VelXAl,         & ! (in)
        & pz_VelXNl,         & ! (inout)
        & pz_VelXBl          & ! (in)
        & )
      call AsselinFilter_xr( &
        & xr_VelZAl,         & ! (in)
        & xr_VelZNl,         & ! (inout)
        & xr_VelZBl          & ! (in)
        & )
      call AsselinFilter_xz( &
        & xz_PotTempAl,      & ! (in)
        & xz_PotTempNl,      & ! (inout)
        & xz_PotTempBl       & ! (in)
        & )
      call AsselinFilter_xz( &
        & xz_KmAl,           & ! (in)
        & xz_KmNl,           & ! (inout)
        & xz_KmBl            & ! (in)
        & )
      call AsselinFilter_xza( &
        & xza_MixRtAl,        & ! (in)
        & xza_MixRtNl,        & ! (inout)
        & xza_MixRtBl         & ! (in)
        & )
    
      ! ݥ.
      ! Numerical dumping.
      !
      call DampSponge_pz( &
        & pz_VelXAl,      & ! (inout)
        & pz_VelXBl,      & ! (inout)
        & DelTimeLFrog(t) & ! (in)
        & )
      call DampSponge_xr( &
        & xr_VelZAl,      & ! (inout)
        & xr_VelZBl,      & ! (inout)
        & DelTimeLFrog(t) & ! (in)
        & )
      call DampSponge_xz( xz_ExnerAl,   xz_ExnerBl,   DelTimeLFrog(t) )
      call DampSponge_xz( xz_PotTempAl, xz_PotTempBl, DelTimeLFrog(t) )
      call DampSponge_xz( xz_KmAl,      xz_KmBl,      DelTimeLFrog(t) )

      ! Ĺ֥ƥåפΥ롼פ󤹤ν.
      ! Renew prognostic variables for next long time step integration.
      !
      xz_PotTempBl = xz_PotTempNl
      xza_MixRtBl  = xza_MixRtNl
      xz_ExnerBl   = xz_ExnerNl
      pz_VelXBl    = pz_VelXNl
      xr_VelZBl    = xr_VelZNl
      xz_KmBl      = xz_KmNl
      xz_KhBl      = xz_KhNl
      
      xz_PotTempNl = xz_PotTempAl
      xza_MixRtNl  = xza_MixRtAl
      xz_ExnerNl   = xz_ExnerAl
      pz_VelXNl    = pz_VelXAl
      xr_VelZNl    = xr_VelZAl
      xz_KmNl      = xz_KmAl
      xz_KhNl      = xz_KhAl
      
    end do
    
    ! ҥȥ꡼եؤν.
    ! Out put to history file.
    !
    call MessageNotify( "M", "main", "Time = %f", d=(/Time/) )
    
    ! ήФ CFL Υå 
    ! CFL condtion check for advection
    !
    call CFLCheckTimeLongVelX( &
      & pz_VelXNl              & ! (in)
      & )
    call CFLCheckTimeLongVelZ( &
      & xr_VelZNl              & ! (in)
      & )

    ! ҥȥե.
    ! Out put to history file.
    !
    call HistoryFile_Output( &
      & Time,                & ! (in)
      & xz_PotTempNl,        & ! (in) 
      & xz_ExnerNl,          & ! (in)
      & pz_VelXNl,           & ! (in)
      & xr_VelZNl,           & ! (in)
      & xza_MixRtNl,         & ! (in)
      & xz_KmNl,             & ! (in)
      & xz_KhNl              & ! (in)
      & )

    xz_VelXNl = xz_avr_pz(pz_VelXNl)
    xz_VelZNl = xz_avr_xr(xr_VelZNl)
    call HistoryAutoPut(Time, 'PotTemp', xz_PotTempNl(FileXMin:FileXMax, FileZMin:FileZMax))
    call HistoryAutoPut(Time, 'Exner',   xz_ExnerNl(FileXMin:FileXMax, FileZMin:FileZMax))
    call HistoryAutoPut(Time, 'VelX',    xz_VelXNl(FileXMin:FileXMax, FileZMin:FileZMax))
    call HistoryAutoPut(Time, 'VelZ',    xz_VelZNl(FileXMin:FileXMax, FileZMin:FileZMax))
    call HistoryAutoPut(Time, 'Km',      xz_KmNl(FileXMin:FileXMax, FileZMin:FileZMax))
    call HistoryAutoPut(Time, 'Kh',      xz_KhNl(FileXMin:FileXMax, FileZMin:FileZMax))
    do s = 1, SpcNum
      call HistoryAutoPut(Time, trim(SpcWetSymbol(s)), xza_MixRtNl(FileXMin:FileXMax, FileZMin:FileZMax, s))
    end do 

    ! ѻͤΥꥢ.
    ! Clear monitor variables.
    !
    call StorePotTempClean
    call StoreMixRtClean
    call StoreMomClean
    call StoreBuoyClean
    call StoreStabClean
  end do
  
  ! ϥեΥ
  ! Close out put files.
  !
  call HistoryFileAutoClose
  call HistoryFile_Close

  ! ꥹȥեκ
  ! Make restartfile.
  !
  call ReStartFile_Open( )
  call ReStartFile_OutPut( &
    & Time - DelTimeLong,  & ! (in)
    & xz_PotTempBl,        & ! (in)
    & xz_ExnerBl,          & ! (in)
    & pz_VelXBl,           & ! (in)
    & xr_VelZBl,           & ! (in)
    & xza_MixRtBl,         & ! (in)
    & xz_KmBl,             & ! (in)
    & xz_KhBl              & ! (in)
    & )
  call ReStartFile_OutPut(                            &
    & Time,                                           &
    & xz_PotTempNl,        & ! (in)
    & xz_ExnerNl,          & ! (in)
    & pz_VelXNl,           & ! (in)
    & xr_VelZNl,           & ! (in)
    & xza_MixRtNl,         & ! (in)
    & xz_KmNl,             & ! (in)
    & xz_KhNl              & ! (in)
    & )
  call ReStartFile_Close

  !----------------------------------------------------
  ! MPI END
  !
#ifdef MPI
  call MPIWrapperFinalize
#endif
  
contains
!-----------------------------------------------------------------------
  subroutine MainInit

    !ۤηػ
    implicit none

    real(8) :: ct
    integer :: s

#ifdef MPI    
    call MPIWrapperInit
#endif
    
    ! NAMELIST ե̾ɤ߹
    ! Loading NAMELIST file.
    !
    call argset_init( &
         & cfgfile       & ! (out) 
         & )

    ! Flag 
    !
    !
    call deepconv_main_nml ( &
         & cfgfile           & ! (in) 
         & )

    ! ǥХåν
    ! Initialization of debug output control.
    !
    call debugset_init( &
         & cfgfile         & ! (in)
         & )
    
    ! ν
    ! Initialization of chemical constatns.
    !
    call chemdata_init( )
    
    ! ʬν
    ! Initialization of time integration.
    !
    call timeset_init( &
         & cfgfile        & ! (in)
         & )
    
    ! ʻν
    ! Initialization of grid arrangement.
    !
    call gridset_init( &
         & cfgfile        & ! (in)
         & )
    
    ! ط׻롼ν
    ! Initialization of chemical routines.
    !
    call chemcalc_init()
    
    ! ܾѿν
    ! Initialization of basic state variables.
    !
    call basicset_init( &
         & cfgfile         & ! (in)
         & )
    
    ! I/O ե̾ν
    ! Initialization of output file name. 
    !
    call fileset_init( &
         & cfgfile        & ! (in)
         & )
    
    ! ͭѿν
    ! Initialization of common variables for moist process.
    !
    call moistset_init( )
    
    ! ѻݴѿν
    ! Initialization of monitor variables.
    !
    call StorePotTemp_init( )
    call StoreMixRt_init( )
    call StoreMom_init( )
    call StoreBuoy_init( )  
    call StoreStab_init( )  

    ! ѿν
    ! Initialization of internal variables.
    !
    call AllocateVariables
    
    ! ͤ 
    ! * ReStartFile ꤵƤˤϥեɤ߹. 
    !   ꤵƤʤˤϥǥեȤδܾȾ. 
    !
    ! Initial value set up.
    ! * Read restartfile if it is specified. If not, make default basic
    !   state and disturbance.
    !
    call MessageNotify( "M", "main", "Initial value setup." )
    
    if (trim(InitFile) /= '') then    
       
       call MessageNotify( &
            & "M", "main", "Restart file is %c", c1=trim(Initfile) )
       
       call ReStartFile_Get( &
            & ReStartTime,      & ! (out)
            & xz_PotTempBl,     & ! (out)
            & xz_ExnerBl,       & ! (out)
            & pz_VelXBl,        & ! (out)
            & xr_VelZBl,        & ! (out)
            & xza_MixRtBl,      & ! (out)
            & xz_KmBl,          & ! (out)
            & xz_KhBl,          & ! (out)
            & xz_PotTempNl,     & ! (out)
            & xz_ExnerNl,       & ! (out)
            & pz_VelXNl,        & ! (out)
            & xr_VelZNl,        & ! (out)
            & xza_MixRtNl,      & ! (out)
            & xz_KmNl,          & ! (out)
            & xz_KhNl           & ! (out)
            & )
    else
       
       call MessageNotify( &
            & "W", "main", "Restart file is not specified." )
       
       call BasicEnv( )
       call DisturbEnv(  &
            & cfgfile,      & ! (out)
            & xz_PotTempBl, & ! (out)
            & xz_ExnerBl,   & ! (out)
            & pz_VelXBl,    & ! (out)
            & xr_VelZBl,    & ! (out)
            & xza_MixRtBl,  & ! (out)
            & xz_KmBl,      & ! (out)
            & xz_KhBl       & ! (out)
            & )
       
       call BoundaryXCyc_pz( pz_VelXBl )     ! (inout)
       call BoundaryZSym_pz( pz_VelXBl )     ! (inout)
       
       call BoundaryXCyc_xr( xr_VelZBl )     ! (inout)
       call BoundaryZAntiSym_xr( xr_VelZBl ) ! (inout)
       
       call BoundaryXCyc_xz( xz_PotTempBl )  ! (inout)
       call BoundaryZSym_xz( xz_PotTempBl )  ! (inout)
       
       call BoundaryXCyc_xza( xza_MixRtBl )  ! (inout)
       call BoundaryZSym_xza( xza_MixRtBl )  ! (inout)
       
       
       !  $ t $ ѿͤνͤ
       ! * 1 롼ܤ $ t $ ͤ $ t-\Delta t$ ͤƱˤ. 
       !   1 ƥåܤϥ顼ˡǲɬפ뤬, 1 ƥåܤ
       !   ʳΥƥåפ̡˥ǥ󥰤ʤ
       !
       ! Set up initial value of time = "t" variables.
       !  
       xz_PotTempNl = xz_PotTempBl
       xz_ExnerNl   = xz_ExnerBl
       pz_VelXNl    = pz_VelXBl
       xr_VelZNl    = xr_VelZBl
       xza_MixRtNl  = xza_MixRtBl
       xz_KmNl      = xz_KmBl
       xz_KhNl      = xz_KhBl    
    end if
    
    ! ໤ν
    ! Initialization of numerical friction coefficient.
    !
    call Damping_Init( &
         & cfgfile        & ! (in)
         & )      
    
    ! ͳȻν
    ! Initialization of numerical diffusion term.
    !
    call NumDiffusion_Init( )          
    
    ! ήȻν
    ! Initialization of turbulent diffusion term.
    !
    call Turbulence_Init( )            
    
    ! ȤΥѥ᥿ꥼν 
    ! Initialization of warmrain parameterization.
    !
    call WarmRainPrm_Init( &
         & cfgfile            & ! (in)
         & )  
    
    ! μ̤Ŷ׻ν
    ! Initialization of negative moist value correction.
    !
    call FillNegative_Init( &
         & xza_MixRtBasicZ,    & ! (in)
         & xz_DensBasicZ       & ! (in)
         & )
    
    ! Ͷν
    !  Initialization of radiative forcing.
    !
    call Radiation_Init( &
         & cfgfile          & ! (in)    
         & )
 
    ! ᵤϷ׻ν
    ! Initialization of moist buoyancy calculation.
    !
    call MoistBuoy_Init( ) 
    
    ! Ϸ׻ѷν 
    ! Initialization of coefficient matrix for exner function calculation.
    !
    call xz_Exner_Init( )              
    
    ! ȥ롼ײν
    ! Initialization of time integration.
    !
    NstepLFrog   = NstepLong 
    NstepEular   = NstepShort 
    DelTimeLFrog = DelTimeLong * 2.0d0 
    DelTimeEular = DelTimeShort
    
    ! ׻ϻȻֳʻҴֳ֤ν
    ! * ReStartFile ꤵƤ, ե뤫ɤ߹ͤ.
    ! * ReStartFile ꤵƤʤ
    !   * ϻ 0.0
    !   * 1 ƥåܤλֳʻҴֳ֤ۤ˻
    !
    ! Setup restart time and time interval. 
    ! * Read restartfile if it is specified.
    ! * If not, 
    !   * "t" is set to be 0.
    !   * Time intervals for 1st step are specified explicitly.
    !
    if ( trim(InitFile) /= '') then    
       Time = ReStartTime(2)               
    else
       Time = 0.0d0                    
       NstepEular(1)   = NstepShort /2 
       DelTimeLFrog(1) = DelTimeLong   
    end if
    
    ! ҥȥ꡼եؤν
    ! Out put to history file.
    !
    call BasicZFileOpen
    call BasicZFilePut
    call BasicZFileClose

    ! ҥȥ꡼եؤν
    ! Out put to history file.
    !
    call HistoryFile_Open( )

    ! ϻ֤Ѱ
    if ( trim(InitFile) /= '') then    
      ct = ReStartTime(2)
    elseif (Time == 0.0) then 
      ct = 0.0d0
    else
      stop
    end if

    ! HistoryCreate
    call HistoryFileAutoOpen(ReStartTime(2), cfgfile)
    call HistoryAutoAddMain
    call HistoryAutoAddFlux
    call HistoryAutoAddBuoy
    call HistoryAutoAddPotTemp
    call HistoryAutoAddMom
    call HistoryAutoAddMixRt

    ! OutPut
    xz_VelXNl = xz_avr_pz(pz_VelXNl)
    xz_VelZNl = xz_avr_xr(xr_VelZNl)
    call HistoryAutoPut(ct, 'PotTemp', xz_PotTempNl(FileXMin:FileXMax, FileZMin:FileZMax))
    call HistoryAutoPut(ct, 'Exner',   xz_ExnerNl(FileXMin:FileXMax, FileZMin:FileZMax))
    call HistoryAutoPut(ct, 'VelX',    xz_VelXNl(FileXMin:FileXMax, FileZMin:FileZMax))
    call HistoryAutoPut(ct, 'VelZ',    xz_VelZNl(FileXMin:FileXMax, FileZMin:FileZMax))
    call HistoryAutoPut(ct, 'Km',      xz_KmNl(FileXMin:FileXMax, FileZMin:FileZMax))
    call HistoryAutoPut(ct, 'Kh',      xz_KhNl(FileXMin:FileXMax, FileZMin:FileZMax))
    do s = 1, SpcNum
      call HistoryAutoPut(ct, trim(SpcWetSymbol(s)), xza_MixRtNl(FileXMin:FileXMax, FileZMin:FileZMax,s))
    end do

    ! ȤФ CFL Υå
    ! CFL condtion check for sound wave.
    !
    call CFLCheckTimeShort( &
         & xz_VelSoundBasicZ   & ! (in)
         & )
    
  end subroutine MainInit


  subroutine deepconv_main_nml ( cfgfile  )

    use dc_iounit,     only : FileOpen    

    implicit none
    character(*), intent(in)      :: cfgfile  ! nml file
    integer                       :: unit     !ֹ

    NAMELIST /deepconv_main_nml / &
      & FlagCondKesslerNH4SH, FlagRadHeatConst, FlagFluxDiff
    
    call FileOpen(unit, file=cfgfile, mode='r')
    read(unit, NML=deepconv_main_nml)
    close(unit)

    write(*,*) "FlagCondKesslerNH4SH ", FlagCondKesslerNH4SH
    write(*,*) "FlagRadHeatConst", FlagRadHeatConst
    write(*,*) "FlagFluxDiff", FlagFluxDiff
    
  end subroutine deepconv_main_nml


  subroutine BasicZFileOpen

    use gtool_history, only: HistoryCreate, HistoryAddVariable
    use fileset,  only: DefaultFilePrefix, &!ҥȥե̾
      &                 exptitle,          &!ǡɽ
      &                 expsrc,            &!ǡ
      &                 expinst,           &!ǽѹԡȿ
      &                 FileNum             !ҥȥեο
    use gridset, only:  FileNX,            &!X γʻ(ե)
      &                 FileNZ              !Z γʻ(ե)
    character(STRING) :: bzfile
    
    bzfile = trim(DefaultFilePrefix)//'BasicZ.nc'

    call HistoryCreate(                              &
      & file = bzfile,                               &
      & title = exptitle,                            &
      & source = expsrc,                             &
      & institution = expinst,                       &
      & dims=(/'x','z','t'/),                        &
      & dimsizes=(/FileNX, FileNZ, 0/),              &
      & longnames=(/'X-coordinate',                  &
      &             'Z-coordinate',                  &
      &             'Time        '/),                &
      & units=(/'m  ','m  ','sec'/), origin=0.0,     &
      & xtypes=(/'double', 'double', 'double'/),     &
      & quiet=.true.,                                &
      & interval=0.0)


    !-----------------------------------------------------------  
    ! ܾν
    !-----------------------------------------------------------  
    !̵Ϥδܾ
    call HistoryAddVariable(  &
      & varname='ExnerBasicZ',&
      & dims=(/'x','z'/),     &
      & longname='nondimensional pressure', units='1',&
      & xtype='double' )
    
    !̤δܾ
    call HistoryAddVariable(  &
      & varname='PotTempBasicZ',&
      & dims=(/'x','z'/),     &
      & longname='potential temperature', &
      & units='K',            &
      & xtype='double' )
    
    !̤δܾ
    call HistoryAddVariable(  &
      & varname='VPotTempBasicZ',&
      & dims=(/'x','z'/),     &
      & longname='vertial potential temperature', &
      & units='K',            &
      & xtype='double' )
    
    !̩٤δܾ
    call HistoryAddVariable(  &
      & varname='DensBasicZ', &
      & dims=(/'x','z'/),     &
      & longname='density',   &
      & units='Kg.m-3',       &
      & xtype='double'  )
    
    !®٤δܾ
    call HistoryAddVariable(  &
      & varname='VelSoundBasicZ',&
      & dims=(/'x','z'/),     &
      & longname='sound velocity',&
      & units='m.s-2',        &
      & xtype='double'  )
    
    !٤δܾ
    call HistoryAddVariable(  &
      & varname='TempBasicZ', &
      & dims=(/'x','z'/),     &
      & longname='Temperature of basic state', &
      & units='K',            &
      & xtype='double'  )
    
    !Ϥδܾ
    call HistoryAddVariable(  &
      & varname='PressBasicZ',&
      & dims=(/'x','z'/),     &
      & longname='Pressure of basic state', &
      & units='Pa',           &
      & xtype='double'  )
    
    !δܾ
    do s = 1, SpcNum
      call HistoryAddVariable(  &
        & varname=trim(SpcWetSymbol(s))//'BasicZ',&
        & dims=(/'x','z'/), &
        & longname=trim(SpcWetSymbol(s))//' Mixing Ratio of basic state', &
        & units='kg.kg-1',        &
        & xtype='double'  )
    end do

    !ʬ̸
    call HistoryAddVariable(  &
      & varname='EffMolWtBasicZ', &
      & dims=(/'x','z'/),     &
      & longname='Effect of Mole Weight', &
      & units='1',            &
      & xtype='double'  )

  end subroutine BasicZFileOpen


  subroutine BasicZFilePut

    use gtool_history, only: HistoryPut
    
    !-------------------------------------------------------------
    ! ܾΥե
    !-------------------------------------------------------------
    call HistoryPut(   &
      & 'DensBasicZ',  &
      & xz_DensBasicZ(FileXMin:FileXMax, FileZMin:FileZMax) )

    call HistoryPut(   &
      & 'ExnerBasicZ', &
      & xz_ExnerBasicZ(FileXMin:FileXMax, FileZMin:FileZMax) )

    call HistoryPut(   &
      & 'PotTempBasicZ',&
      & xz_PotTempBasicZ(FileXMin:FileXMax, FileZMin:FileZMax) )

    call HistoryPut(   &
      & 'VPotTempBasicZ',&
      & xz_PotTempBasicZ(FileXMin:FileXMax, FileZMin:FileZMax)    &
      & / xz_EffMolWtBasicZ(FileXMin:FileXMax, FileZMin:FileZMax) )

    call HistoryPut(   &
      & 'VelSoundBasicZ', &
      & xz_VelSoundBasicZ(FileXMin:FileXMax, FileZMin:FileZMax) )

    call HistoryPut(   &
      & 'TempBasicZ',  &
      & xz_TempBasicZ(FileXMin:FileXMax, FileZMin:FileZMax) )

    call HistoryPut(   &
      & 'PressBasicZ', &
      & xz_PressBasicZ(FileXMin:FileXMax, FileZMin:FileZMax) )

    call HistoryPut(   &
      & 'EffMolWtBasicZ', &
      & xz_EffMolWtBasicZ(FileXMin:FileXMax, FileZMin:FileZMax) )

    do s = 1, SpcNum
      call HistoryPut( &
        & trim(SpcWetSymbol(s))//'BasicZ',    &
        & xza_MixRtBasicZ(FileXMin:FileXMax, FileZMin:FileZMax, s) )
    end do

  end subroutine BasicZFilePut
  

  subroutine BasicZFileClose

    use gtool_history, only: HistoryClose

    call HistoryClose
    
  end subroutine BasicZFileClose



  !-----------------------------------------------------------------------
  subroutine HistoryAutoAddMain
    !
    ! ͽѿν
    !
    use gtool_historyauto, only: HistoryAutoAddVariable

    integer :: s

    !̵Ϥξ
    call HistoryAutoAddVariable(  &
      & varname='Exner',      &  
      & dims=(/'x','z','t'/), &
      & longname='disturbunce of nondimensional pressure', &
      & units=' ',            &        
      & xtype='double' )
    
    !̤ξ
    call HistoryAutoAddVariable(  &
      & varname='PotTemp',    &
      & dims=(/'x','z','t'/), &
      & longname='disturbunce of potential temperature',  &
      & units='K',            &
      & xtype='double' )

    !ʿ®
    call HistoryAutoAddVariable(  &
      & varname='VelX',       &
      & dims=(/'x','z','t'/), &
      & longname='zonal velocity', &
      & units='m.s-1',      &
      & xtype='double' )

    !ľ®
    call HistoryAutoAddVariable(  &
      & varname='VelZ',       &
      & dims=(/'x','z','t'/), &
      & longname='vertical velocity', &
      & units='m.s-1',      &
      & xtype='double' )

    !Ǵ(ư)
    call HistoryAutoAddVariable(  &
      & varname='Km',         &
      & dims=(/'x','z','t'/), &
      & longname='turbulet diffusion coefficient', &
      & units='m2.s-1',            &
      & xtype='double' )
  
    !Ǵ(Ǯ)
    call HistoryAutoAddVariable(  &
      & varname='Kh',         &
      & dims=(/'x','z','t'/), &
      & longname='turbulet diffusion coefficient for heat', &
      & units='m2.s-1',            &
      & xtype='double')
  
    !
    do s = 1, SpcNum
      call HistoryAutoAddVariable(                               &
        & varname=trim(SpcWetSymbol(s)),                     &
        & dims=(/'x','z','t'/),                              &
        & longname=trim(SpcWetSymbol(s))//' Mixing Ratio',   &
        & units='kg.kg-1',                                 &
        & xtype='double')
    end do

  end subroutine HistoryAutoAddMain


  subroutine HistoryAutoAddFlux

    use gtool_historyauto, only: HistoryAutoAddVariable

    !ɽ̥եå
    call HistoryAutoAddVariable(  &
      & varname='MixRtFluxBulk',         &
      & dims=(/'x','z','t'/), &
      & longname='flux of mixing ratio of vapor from surface', &
      & units='kg.kg-1.s-1',            &
      & xtype='double')

    !ɽ̥եå(Ǯ)
    call HistoryAutoAddVariable(  &
      & varname='HeatFluxBulk',         &
      & dims=(/'x','z','t'/), &
      & longname='flux of heating from surface', &
      & units='K.s-1',            &
      & xtype='double')

    !ɽ̥եå(ư)
    call HistoryAutoAddVariable(  &
      & varname='MomFluxBulk',         &
      & dims=(/'x','z','t'/), &
      & longname='flux of momentum from surface', &
      & units='kg.m.s-1',            &
      & xtype='double')

  end subroutine HistoryAutoAddFlux

  subroutine HistoryAutoAddBuoy

    use gtool_historyauto, only: HistoryAutoAddVariable

    call HistoryAutoAddVariable(  &
      & varname='BuoyTemp', &
      & dims=(/'x','z','t'/),     &
      & longname='Buoyancy (Temp)',  &
      & units='s-2',    &
      & xtype='double')

    call HistoryAutoAddVariable(  &
      & varname='BuoyMolWt', &
      & dims=(/'x','z','t'/),     &
      & longname='Buoyancy (MolWt)',  &
      & units='s-2',    &
      & xtype='double' )

    call HistoryAutoAddVariable(  &
      & varname='BuoyDrag', &
      & dims=(/'x','z','t'/),     &
      & longname='Buoyance (Drag)',  &
      & units='s-2',    &
      & xtype='double' )

  end subroutine HistoryAutoAddBuoy


  subroutine HistoryAutoAddPotTemp

    use gtool_historyauto, only: HistoryAutoAddVariable

    call HistoryAutoAddVariable(  &
      & varname='PotTempAdv', &
      & dims=(/'x','z','t'/),     &
      & longname='Advection term of potential temperature',  &
      & units='K.s-1',    &
      & xtype='double')

    call HistoryAutoAddVariable(  &
      & varname='PotTempTurb',&
      & dims=(/'x','z','t'/),     &
      & longname='Turbulence term of potential temperature', &
      & units='K.s-1"',    &
      & xtype='double')

    call HistoryAutoAddVariable(  &
      & varname='PotTempDisp',&
      & dims=(/'x','z','t'/),     &
      & longname='Dissipation term of potential temperature', &
      & units='K.s-1"',    &
      & xtype='double')

    call HistoryAutoAddVariable(  &
      & varname='PotTempRad', &
      & dims=(/'x','z','t'/),     &
      & longname='Radiation term of potential temperature', &
      & units='K.s-1"',    &
      & xtype='double')
    
    call HistoryAutoAddVariable(  &
      & varname='PotTempDiff',&
      & dims=(/'x','z','t'/),     &
      & longname='Numerical diffusion term of potential temperature',&
      & units='K.s-1',    &
      & xtype='double')
    
    call HistoryAutoAddVariable(  &
      & varname='PotTempCond',&
      & dims=(/'x','z','t'/),     &
      & longname='Latent heat term of potential temperature', &
      & units='K.s-1',    &
      & xtype='double')

    call HistoryAutoAddVariable(  &
      & varname='PotTempFlux',&
      & dims=(/'x','z','t'/),     &
      & longname='Surface Flux term of potential temperature', &
      & units='K.s-1"',    &
      & xtype='double')

    call HistoryAutoAddVariable(  &
      & varname='PotTempDamp',&
      & dims=(/'x','z','t'/),     &
      & longname='Newtonian Cooling term of potential temperature',  &
      & units='K.s-1"',    &
      & xtype='double')

  end subroutine HistoryAutoAddPotTemp


  subroutine HistoryAutoAddMom

    use gtool_historyauto, only: HistoryAutoAddVariable

    call HistoryAutoAddVariable(  &
      & varname='Mom_Adv',&
      & dims=(/'x','z','t'/),     &
      & longname='Advection term of momentum in x direction',  &
      & units='kg.m..s-2"',    &
      & xtype='double')

    call HistoryAutoAddVariable(  &
      & varname='Mom_Turb',&
      & dims=(/'x','z','t'/),     &
      & longname='Turblence term of momentum in x direction',  &
      & units='kg.m..s-2"',    &
      & xtype='double')

    call HistoryAutoAddVariable(  &
      & varname='Mom_Diff',&
      & dims=(/'x','z','t'/),     &
      & longname='Numerical diffusion term of momentum in x direction',  &
      & units='kg.m..s-2"',    &
      & xtype='double')

    call HistoryAutoAddVariable(  &
      & varname='Mom_Flux',&
      & dims=(/'x','z','t'/),     &
      & longname='Flux term of momentum in x direction',  &
      & units='kg.m..s-2"',    &
      & xtype='double')
    
  end subroutine HistoryAutoAddMom


  subroutine HistoryAutoAddMixRt

    use gtool_historyauto, only: HistoryAutoAddVariable

    integer :: s

    do s = 1, SpcNum
      call HistoryAutoAddVariable(  &
        & varname=trim(SpcWetSymbol(s))//'_Adv', &
        & dims=(/'x','z','t'/),     &
        & longname='Advection term of '          &
        &           //trim(SpcWetSymbol(s))//' mixing ratio',  &
        & units='kg.kg-1.s-1',    &
        & xtype='double')
      
      call HistoryAutoAddVariable(  &
        & varname=trim(SpcWetSymbol(s))//'_Turb', & 
        & dims=(/'x','z','t'/),     &
        & longname='Turbulence term of '          &
        &           //trim(SpcWetSymbol(s))//' mixing ratio',  &
        & units='kg.kg-1.s-1',    &
        & xtype='double')
      
      call HistoryAutoAddVariable(  &
        & varname=trim(SpcWetSymbol(s))//'_Diff', & 
        & dims=(/'x','z','t'/),     &
        & longname='Diffusion term of '          &
        &           //trim(SpcWetSymbol(s))//' mixing ratio',  &
        & units='kg.kg-1.s-1',    &
        & xtype='double')
      
      call HistoryAutoAddVariable(  &
        & varname=trim(SpcWetSymbol(s))//'_Flux', & 
        & dims=(/'x','z','t'/),     &
        & longname='Surface Flux term of '          &
        &           //trim(SpcWetSymbol(s))//' mixing ratio',  &
        & units='kg.kg-1.s-1',    &
        & xtype='double')
    
      call HistoryAutoAddVariable(  &
        & varname=trim(SpcWetSymbol(s))//'_Rain', & 
        & dims=(/'x','z','t'/),     &
        & longname='Fall Rain term of '          &
        &           //trim(SpcWetSymbol(s))//' mixing ratio',  &
        & units='kg.kg-1.s-1',    &
        & xtype='double')
    
      call HistoryAutoAddVariable(  &
        & varname=trim(SpcWetSymbol(s))//'_Fill', & 
        & dims=(/'x','z','t'/),     &
        & longname='Filling Negative term 1 of '          &
        &           //trim(SpcWetSymbol(s))//' mixing ratio',  &
        & units='kg.kg-1.s-1',    &
        & xtype='double')
    
      call HistoryAutoAddVariable(  &
        & varname=trim(SpcWetSymbol(s))//'_Cond', & 
        & dims=(/'x','z','t'/),     &
        & longname='Condensation term of '          &
        &           //trim(SpcWetSymbol(s))//' mixing ratio',  &
        & units='kg.kg-1.s-1',    &
        & xtype='double')

    end do
    
  end subroutine HistoryAutoAddMixRt

  subroutine TendencyKm_Adv_Diff
    ! ȻΰήȻ
    ! Advection and diffusion of turbulent diffusion coefficient.
    !

    real(8) :: xz_Adv(DimXMin:DimXMax, DimZMin:DimZMax)
    real(8) :: xz_Shr(DimXMin:DimXMax, DimZMin:DimZMax)
    real(8) :: xz_Diff(DimXMin:DimXMax, DimZMin:DimZMax)
    real(8) :: xz_Buoy(DimXMin:DimXMax, DimZMin:DimZMax)
    real(8) :: xz_Disp(DimXMin:DimXMax, DimZMin:DimZMax)

    xz_Adv  = xz_AdvKm(xz_KmNl, pz_VelXNl, xr_VelZNl)
    xz_Buoy = xz_BuoyMoistKm(xz_PotTempBl, xz_ExnerBl, xza_MixRtBl)
    xz_Shr  = xz_ShearKm(xz_KmBl, pz_VelXBl, xr_VelZBl) 
    xz_Diff = xz_NumDiffKm(xz_KmBl)
    xz_Disp = xz_DispKm(xz_KmBl) 
    
    xz_KmTend = xz_Adv + xz_Buoy + xz_Shr + xz_Diff + xz_Disp 
        
  end subroutine TendencyKm_Adv_Diff


  subroutine TendencyPotTemp_Adv_Diff

    real(8) :: xz_Adv(DimXMin:DimXMax, DimZMin:DimZMax)
    real(8) :: xz_Diff(DimXMin:DimXMax, DimZMin:DimZMax)
    real(8) :: xz_Turb(DimXMin:DimXMax, DimZMin:DimZMax)
    real(8) :: xz_Disp(DimXMin:DimXMax, DimZMin:DimZMax)

    ! ̤ΰήȻη׻ 
    ! Advection and diffusion of potential temperature.
    xz_Adv  = xz_AdvScalar( xz_PotTempNl + xz_PotTempBasicZ, pz_VelXNl, xr_VelZNl)
    xz_Turb = xz_TurbScalar(xz_PotTempBl + xz_PotTempBasicZ, xz_KhBl)
    xz_Diff = xz_NumDiffScalar(xz_PotTempBl)
    xz_Disp = xz_DispHeat( xz_KmBl )                               

    xz_PotTempTend = xz_Adv + xz_Turb + xz_Diff + xz_Disp

    ! Output
    call HistoryAutoPut(Time, 'PotTempAdv',  xz_Adv(FileXMin:FileXMax, FileZMin:FileZMax))
    call HistoryAutoPut(Time, 'PotTempDiff', xz_Diff(FileXMin:FileXMax, FileZMin:FileZMax))
    call HistoryAutoPut(Time, 'PotTempTurb', xz_Turb(FileXMin:FileXMax, FileZMin:FileZMax))
    call HistoryAutoPut(Time, 'PotTempDisp', xz_Disp(FileXMin:FileXMax, FileZMin:FileZMax))

  end subroutine TendencyPotTemp_Adv_Diff


  subroutine TendencyMixRt_Adv_Diff

    real(8) :: xza_Adv(DimXMin:DimXMax, DimZMin:DimZMax, SpcNum)
    real(8) :: xza_Diff(DimXMin:DimXMax, DimZMin:DimZMax, SpcNum)
    real(8) :: xza_Turb(DimXMin:DimXMax, DimZMin:DimZMax, SpcNum)
    real(8) :: xza_Fall(DimXMin:DimXMax, DimZMin:DimZMax, SpcNum)

    xza_Fall = 0.0d0
    xza_Adv  = xza_AdvScalar(xza_MixRtNl + xza_MixRtBasicZ,  pz_VelXNl, xr_VelZNl) 
    xza_Turb = xza_TurbScalar(xza_MixRtBl + xza_MixRtBasicZ, xz_KhBl) 
    xza_Diff = xza_NumDiffScalar(xza_MixRtBl) 
    xza_Fall = xza_FallRain(xza_MixRtBl) 
    
    ! ŽʬΰήȻ 
    ! Advection and diffusion of vapor, cloud and rain mixing ratios.
    xza_MixRtTend = xza_Adv + xza_Turb + xza_Diff + xza_Fall
    
    ! Output
    do s = 1, SpcNum
      call HistoryAutoPut(Time, trim(SpcWetSymbol(s))//'_Adv',  xza_Adv(FileXMin:FileXMax, FileZMin:FileZMax, s))
      call HistoryAutoPut(Time, trim(SpcWetSymbol(s))//'_Diff', xza_Diff(FileXMin:FileXMax, FileZMin:FileZMax, s))
      call HistoryAutoPut(Time, trim(SpcWetSymbol(s))//'_Turb', xza_Turb(FileXMin:FileXMax, FileZMin:FileZMax, s))
      call HistoryAutoPut(Time, trim(SpcWetSymbol(s))//'_Rain', xza_Fall(FileXMin:FileXMax, FileZMin:FileZMax, s))
    end do
    
  end subroutine TendencyMixRt_Adv_Diff


  subroutine AdjustMixRt_FillNegative

    real(8) :: xza_Del(DimXMin:DimXMax, DimZMin:DimZMax, SpcNum)
    real(8) :: xza_MixRtOrg(DimXMin:DimXMax, DimZMin:DimZMax, SpcNum)
    real(8) :: xza_MixRtWork(DimXMin:DimXMax, DimZMin:DimZMax, SpcNum)

    ! ͤݴ
    xza_MixRtOrg = xza_MixRtAl
    
    ! ήˤäˤʤäʬ
    ! Negative values due to advection are corrected.
    !
    xza_MixRtWork = xza_MixRtAl
    xza_MixRtAl = xza_FillNegative_xza( xza_MixRtWork ) 
    
    ! ڤʤäʬ򥼥ˤ. 
    ! Negative values mixing ratios are corrected.
    !
    xza_MixRtWork = xza_MixRtAl  
    xza_MixRtAl = max( - xza_MixRtBasicZ, xza_MixRtWork )
    
    ! ᤿/ä̤ݴ
    ! Adjustion value is stored.
    !    
    xza_Del = xza_MixRtAl - xza_MixRtOrg

    ! Output
    call StoreMixRtFill1( xza_Del(FileXMin:FileXMax, FileZMin:FileZMax, :) / DelTimeLFrog(t))
    do s = 1, SpcNum
      call HistoryAutoPut(Time, trim(SpcWetSymbol(s))//'_Fill', xza_Del(FileXMin:FileXMax, FileZMin:FileZMax, s) / DelTimeLFrog(t))
    end do

  end subroutine AdjustMixRt_FillNegative


  subroutine AdjustMixRt_Kessler
    
    real(8) :: xz_PotTempOrg(DimXMin:DimXMax, DimZMin:DimZMax)
    real(8) :: xz_PotTempWork(DimXMin:DimXMax, DimZMin:DimZMax)
    real(8) :: xza_MixRtOrg(DimXMin:DimXMax, DimZMin:DimZMax, SpcNum)
    real(8) :: xza_MixRtWork(DimXMin:DimXMax, DimZMin:DimZMax, SpcNum)
    real(8) :: xza_Cond(DimXMin:DimXMax, DimZMin:DimZMax, SpcNum)
    real(8) :: xza_Del(DimXMin:DimXMax, DimZMin:DimZMax, SpcNum)
    real(8) :: xz_Cond(DimXMin:DimXMax, DimZMin:DimZMax)
    real(8) :: xz_Del(DimXMin:DimXMax, DimZMin:DimZMax)
    
    ! ޤǤͤݴ
    ! Previous values are stored to work area.
    !
    xz_PotTempOrg = xz_PotTempAl
    xza_MixRtOrg  = xza_MixRtAl
    
    !--------------------------------------------
    ! ȤΥѥ᥿ꥼ.
    !   <--> ѴԤ
    !
    !鱫ؤѴʬɲä. 
    !  xza_Cloud2Rain ؿ, Ȥƻֹߤ뤳Ȥ, 
    !  ʬͤϤ. 
    
    !ޤǤͤݴ
    xza_MixRtWork = xza_MixRtAl
    
    !ؤѲ̤׻
    xza_MixRtAl = xza_MixRtWork + xza_Cloud2Rain( xza_MixRtAl, DelTimeLFrog(t) )
    
    !
    call BoundaryXCyc_xza( xza_MixRtAl )
    call BoundaryZSym_xza( xza_MixRtAl )
    
    !--------------------------------------
    ! ˰Ĵ
    ! * <-->ѴԤ.
    !
    ! Moist adjustment.
    ! * Conversion from vapor to cloud.
   
    ! ޤǤͤݴ
    ! Previous values are stored to work area.
    !
    xz_PotTempWork = xz_PotTempAl
    xza_MixRtWork  = xza_MixRtAl

    ! ˰Ĵ
    ! * <-->ѴԤ.
    !
    ! Moist adjustment.
    ! * Conversion from vapor to cloud.
    call MoistAdjustSvapPress(         &
      & xz_ExnerNl,                    & ! (in)
      & xz_PotTempAl, xza_MixRtAl      & ! (inout)
      & )

    if (FlagCondKesslerNH4SH) then 
      call MoistAdjustNH4SH(           &
        & xz_ExnerNl,                  & ! (in)
        & xz_PotTempAl, xza_MixRtAl )    ! (inout)
    end if

    !
    call BoundaryXCyc_xz( xz_PotTempAl )
    call BoundaryZSym_xz( xz_PotTempAl )
    call BoundaryXCyc_xza( xza_MixRtAl )
    call BoundaryZSym_xza( xza_MixRtAl )

    !--------------------------------------  
    ! ȤΥѥ᥿ꥼ.
    ! * <--> ѴԤ
    !
    ! Warm rain parameterization.
    ! * Conversion from rain to vapor.
    
    ! ޤǤͤݴ
    ! Previous values are stored to work area.
    !
    xz_PotTempWork = xz_PotTempAl
    xza_MixRtWork = xza_MixRtAl
    
    ! ؤκѲ
    ! * ̤η׻ˤ, ѲɬפȤʤ뤿, 
    !   Ѳ 1 ĤȤѰդ.
    !
    ! Conversion values are calculated.
    !
    if (FlagCondKesslerNH4SH) then 
      xza_Cond =                                       &
        & (                                            &
        &   + xza_Rain2Gas(                            &
        &        xz_ExnerNl, xz_PotTempAl, xza_MixRtAl, DelTimeLFrog(t) &
        &       )                                      &
        &   + xza_Rain2GasNH4SH(                       &
        &        xz_ExnerNl, xz_PotTempAl, xza_MixRtAl, DelTimeLFrog(t) &
        &       )                                      &
        &  )    
    else
      xza_Cond = xza_Rain2Gas(                         &
        &        xz_ExnerNl, xz_PotTempAl, xza_MixRtAl, DelTimeLFrog(t) &
        &      )
    end if

    ! ̤η׻. ؤѴȼǮȿǮɲ.
    !
    !
    if (FlagCondKesslerNH4SH) then 
      xz_Cond = xz_Rain2GasHeat( xz_PotTempAl, xz_ExnerNl, xza_Cond ) &
        &     + xz_Rain2GasHeatNH4SH(xz_ExnerNl, xza_Cond )           
    else
      xz_Cond = xz_Rain2GasHeat( xz_PotTempAl, xz_ExnerNl, xza_Cond ) 
    end if
      
    ! ̤Ⱥη׻. ؤѴʬɲ
    !
    !
    xz_PotTempAl = xz_PotTempWork + xz_Cond
    xza_MixRtAl  = xza_MixRtWork  + xza_Cond

    ! ʬѰ
    !
    !
    xz_Del  = xz_PotTempAl - xz_PotTempOrg
    xza_Del = xza_MixRtAl  - xza_MixRtOrg
    
    ! ͤݴ
    !
    !
    call StorePotTempCond( &
      & xz_Del / DelTimeLFrog(t) & ! (in)
      & ) 
    call StoreMixRtCond( &
      & xza_Del / DelTimeLFrog(t) & ! (in)
      & ) 
    call HistoryAutoPut(Time, 'PotTempCond', xz_Del(FileXMin:FileXMax, FileZMin:FileZMax) / DelTimeLFrog(t))
    do s = 1, SpcNum
      call HistoryAutoPut(Time, trim(SpcWetSymbol(s))//'_Cond', xza_Del(FileXMin:FileXMax, FileZMin:FileZMax, s) / DelTimeLFrog(t))
    end do
    
    !
    call BoundaryXCyc_xz( xz_PotTempAl )
    call BoundaryZSym_xz( xz_PotTempAl )
    call BoundaryXCyc_xza( xza_MixRtAl )
    call BoundaryZSym_xza( xza_MixRtAl )
    
  end subroutine AdjustMixRt_Kessler


  subroutine TendencyVelX_Adv_Diff
    
    real(8) :: pz_Adv(DimXMin:DimXMax, DimZMin:DimZMax)
    real(8) :: pz_Turb(DimXMin:DimXMax, DimZMin:DimZMax)
    real(8) :: pz_Diff(DimXMin:DimXMax, DimZMin:DimZMax)
    
    pz_Adv = 0.0d0
    pz_Turb = 0.0d0
    pz_Diff = 0.0d0
    
    pz_Adv  = pz_AdvVelX(pz_VelXNl, xr_VelZNl)
    pz_Turb = pz_TurbVelX(xz_KmBl, pz_VelXBl, xr_VelZBl) 
    pz_Diff = pz_NumDiffVelX(pz_VelXBl)
    
    pz_AccelVelXNl = pz_Adv + pz_Turb + pz_Diff
    
  end subroutine TendencyVelX_Adv_Diff
  

  subroutine TendencyVelZ_Adv_Diff

    real(8) :: xr_Adv(DimXMin:DimXMax, DimZMin:DimZMax)
    real(8) :: xr_BuoyT(DimXMin:DimXMax, DimZMin:DimZMax)
    real(8) :: xr_BuoyM(DimXMin:DimXMax, DimZMin:DimZMax)
    real(8) :: xr_BuoyD(DimXMin:DimXMax, DimZMin:DimZMax)
    real(8) :: xr_Turb(DimXMin:DimXMax, DimZMin:DimZMax)
    real(8) :: xr_Diff(DimXMin:DimXMax, DimZMin:DimZMax)

    xr_Adv = 0.0d0
    xr_Turb = 0.0d0
    xr_Diff = 0.0d0
    xr_BuoyT = 0.0d0
    xr_BuoyM = 0.0d0
    xr_BuoyD = 0.0d0

    xr_Adv   = xr_AdvVelZ(xr_VelZNl, pz_VelXNl)              
    xr_BuoyT = xr_Buoy(xz_PotTempNl)                             
    xr_BuoyM = xr_BuoyMolWt(xza_MixRtNl)                     
    xr_BuoyD = xr_BuoyDrag(xza_MixRtNl)                      
    xr_Turb  = xr_TurbVelZ(xz_KmBl, pz_VelXBl, xr_VelZBl)  
    xr_Diff  = xr_NumDiffVelZ(xr_VelZBl)

    xr_AccelVelZNl = xr_Adv + xr_Turb + xr_Diff + xr_BuoyT + xr_BuoyM + xr_BuoyD

  end subroutine TendencyVelZ_Adv_Diff


  subroutine TendencyVelX_PGrd
    
    pz_VelXPGrd = - pz_GradPi(xz_ExnerNs, pz_VelXNs, xr_VelZNs)

  end subroutine TendencyVelX_PGrd


  subroutine TendencyVelZ_PGrd
     
    xr_VelZPGrd = - xr_GradPi(xz_ExnerAs,xz_ExnerNs,pz_VelXNs,xr_VelZNs) 
    
  end subroutine TendencyVelZ_PGrd


  subroutine TendencyPotTemp_Radiation
    
    if (FlagRadHeatConst) then 
      xz_PotTempTendRad = xz_RadHeatConst( xz_ExnerBl ) 
    else
      xz_PotTempTendRad = 0.0d0
    end if

  end subroutine TendencyPotTemp_Radiation


  subroutine TendencyPotTemp_Flux

    if (FlagFluxDiff) then 
      xz_PotTempTendFlux = xz_HeatFluxDiff( xz_PotTempNl )  
    else
      xz_PotTempTendFlux = 0.0d0
    end if

  end subroutine TendencyPotTemp_Flux


  subroutine TendencyMixRt_Flux

    if (FlagFluxDiff) then     
      xza_MixRtTendFlux = xza_MixRtFluxDiff(xza_MixRtNl) 
    else
      xza_MixRtTendFlux = 0.0d0
    end if

  end subroutine TendencyMixRt_Flux

  
  subroutine AllocateVariables
    allocate(                                                  &
      & pz_VelXPGrd(DimXMin:DimXMax, DimZMin:DimZMax),             &
      & xr_VelZPGrd(DimXMin:DimXMax, DimZMin:DimZMax),             &
      & pz_VelXTend(DimXMin:DimXMax, DimZMin:DimZMax),             &
      & xr_VelZTend(DimXMin:DimXMax, DimZMin:DimZMax),             &
      & xz_KmTend(DimXMin:DimXMax, DimZMin:DimZMax),             &
      & xz_PotTempTend(DimXMin:DimXMax, DimZMin:DimZMax),             &
      & xz_PotTempTendRad(DimXMin:DimXMax, DimZMin:DimZMax),             &
      & xz_PotTempTendFlux(DimXMin:DimXMax, DimZMin:DimZMax),             &
      & xza_MixRtTend(DimXMin:DimXMax, DimZMin:DimZMax, SpcNum),    &
      & xza_MixRtTendFlux(DimXMin:DimXMax, DimZMin:DimZMax, SpcNum),    &
      & pz_VelXBl(DimXMin:DimXMax, DimZMin:DimZMax),           &
      & pz_VelXNl(DimXMin:DimXMax, DimZMin:DimZMax),           &
      & xz_VelXNl(DimXMin:DimXMax, DimZMin:DimZMax),           &
      & pz_VelXAl(DimXMin:DimXMax, DimZMin:DimZMax),           &
      & pz_VelXNs(DimXMin:DimXMax, DimZMin:DimZMax),           &
      & pz_VelXAs(DimXMin:DimXMax, DimZMin:DimZMax),           &
!
      & xr_VelZBl(DimXMin:DimXMax, DimZMin:DimZMax),           &
      & xr_VelZNl(DimXMin:DimXMax, DimZMin:DimZMax),           &
      & xz_VelZNl(DimXMin:DimXMax, DimZMin:DimZMax),           &
      & xr_VelZAl(DimXMin:DimXMax, DimZMin:DimZMax),           &
      & xr_VelZNs(DimXMin:DimXMax, DimZMin:DimZMax),           &
      & xr_VelZAs(DimXMin:DimXMax, DimZMin:DimZMax),           &
!
      & xz_ExnerBl(DimXMin:DimXMax, DimZMin:DimZMax),          &
      & xz_ExnerNl(DimXMin:DimXMax, DimZMin:DimZMax),          &
      & xz_ExnerAl(DimXMin:DimXMax, DimZMin:DimZMax),          &
      & xz_ExnerNs(DimXMin:DimXMax, DimZMin:DimZMax),          &
      & xz_ExnerAs(DimXMin:DimXMax, DimZMin:DimZMax),          &
!
      & xz_PotTempAdvBZ(DimXMin:DimXMax, DimZMin:DimZMax),      &
      & xz_PotTempTurbBZ(DimXMin:DimXMax, DimZMin:DimZMax),     &
      & xz_PotTempBl(DimXMin:DimXMax, DimZMin:DimZMax),        &
      & xz_PotTempNl(DimXMin:DimXMax, DimZMin:DimZMax),        &
      & xz_PotTempAl(DimXMin:DimXMax, DimZMin:DimZMax),        &
!
      & xz_KmBl(DimXMin:DimXMax, DimZMin:DimZMax),             &
      & xz_KmNl(DimXMin:DimXMax, DimZMin:DimZMax),             &
      & xz_KmAl(DimXMin:DimXMax, DimZMin:DimZMax),             &
!
      & xz_KhBl(DimXMin:DimXMax, DimZMin:DimZMax),             &
      & xz_KhNl(DimXMin:DimXMax, DimZMin:DimZMax),             &
      & xz_KhAl(DimXMin:DimXMax, DimZMin:DimZMax),             &
!
      & xza_MixRtAdvBZ(DimXMin:DimXMax, DimZMin:DimZMax, SpcNum),  &
      & xza_MixRtTurbBZ(DimXMin:DimXMax, DimZMin:DimZMax, SpcNum), &
      & xza_MixRtBl(DimXMin:DimXMax, DimZMin:DimZMax, SpcNum),   &
      & xza_MixRtNl(DimXMin:DimXMax, DimZMin:DimZMax, SpcNum),   &
      & xza_MixRtAl(DimXMin:DimXMax, DimZMin:DimZMax, SpcNum),   &
!
      & pz_AccelVelXNl(DimXMin:DimXMax, DimZMin:DimZMax),        &
      & xr_AccelVelZNl(DimXMin:DimXMax, DimZMin:DimZMax),        & 
!
      & DelTimeLFrog(NstepLong), NStepEular(NStepLong)         &
      & ) 
    
    pz_VelXNs = 0.0d0;     pz_VelXAs = 0.0d0    
    xr_VelZNs = 0.0d0;     xr_VelZAs = 0.0d0
    xz_ExnerNs = 0.0d0;    xz_ExnerAs = 0.0d0

    pz_VelXBl = 0.0d0;     pz_VelXNl = 0.0d0;     pz_VelXAl = 0.0d0
    xr_VelZBl = 0.0d0;     xr_VelZNl = 0.0d0;     xr_VelZAl = 0.0d0
    xz_ExnerBl = 0.0d0;    xz_ExnerNl = 0.0d0;    xz_ExnerAl = 0.0d0
    xz_KmBl = 0.0d0;       xz_KmNl = 0.0d0;       xz_KmAl = 0.0d0
    xz_KhBl = 0.0d0;       xz_KhNl = 0.0d0;       xz_KhAl = 0.0d0
    xz_PotTempBl = 0.0d0;  xz_PotTempNl = 0.0d0;  xz_PotTempAl = 0.0d0
    xza_MixRtBl = 0.0d0;    xza_MixRtNl = 0.0d0;    xza_MixRtAl = 0.0d0

    pz_AccelVelXNl = 0.0d0
    xr_AccelVelZNl = 0.0d0

    DelTimeEular = 0.0d0
    DelTimeLFrog = 0.0d0; 
    NStepEular = 0.0d0
    NstepLFrog = 0.0d0

  end subroutine AllocateVariables


end program arare
